from __future__ import annotations
from ..Base import *
from ..Proxy import *



class AcadSectionTypeSettings(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)
    
    BackgroundLinesColor: AcadAcCmColor = proxy_property('AcadAcCmColor','BackgroundLinesColor',AccessMode.ReadWrite)
    BackgroundLinesHiddenLine: bool = proxy_property(bool,'BackgroundLinesHiddenLine',AccessMode.ReadWrite)
    BackgroundLinesLayer: str = proxy_property(str,'BackgroundLinesLayer',AccessMode.ReadWrite)
    BackgroundLinesLinetypeScale: float = proxy_property(float,'BackgroundLinesLinetypeScale',AccessMode.ReadWrite)
    BackgroundLinesLineweight: AcLineWeight = proxy_property('AcLineWeight','BackgroundLinesLineweight',AccessMode.ReadWrite)
    BackgroundLinesPlotStyleName: str = proxy_property(str,'BackgroundLinesPlotStyleName',AccessMode.ReadWrite)
    BackgroundLinesVisible: bool = proxy_property(bool,'BackgroundLinesVisible',AccessMode.ReadWrite)
    CurveTangencyLinesColor: AcadAcCmColor = proxy_property('AcadAcCmColor','CurveTangencyLinesColor',AccessMode.ReadWrite)
    CurveTangencyLinesLayer: str = proxy_property(str,'CurveTangencyLinesColor',AccessMode.ReadWrite)
    CurveTangencyLinesLinetype: str = proxy_property(str,'CurveTangencyLinesLinetype',AccessMode.ReadWrite)
    CurveTangencyLinesLinetypeScale: float = proxy_property(float,'CurveTangencyLinesLinetypeScale',AccessMode.ReadWrite)
    CurveTangencyLinesLineweight: AcLineWeight = proxy_property('AcLineWeight','CurveTangencyLinesLineweight',AccessMode.ReadWrite)
    CurveTangencyLinesPlotStyleName: str = proxy_property(str,'CurveTangencyLinesPlotStyleName',AccessMode.ReadWrite)
    CurveTangencyLinesVisible: bool = proxy_property(bool,'CurveTangencyLinesVisible',AccessMode.ReadWrite)
    DestinationBlock: Variant = proxy_property('Variant','DestinationBlock',AccessMode.ReadWrite)
    DestinationFile: str = proxy_property(str,'DestinationFile',AccessMode.ReadWrite)
    ForegroundLinesColor: AcadAcCmColor = proxy_property('AcadAcCmColor','ForegroundLinesColor',AccessMode.ReadWrite)
    ForegroundLinesEdgeTransparency: int = proxy_property(int,'ForegroundLinesEdgeTransparency',AccessMode.ReadWrite)
    ForegroundLinesFaceTransparency: int = proxy_property(int,'ForegroundLinesFaceTransparency',AccessMode.ReadWrite)
    ForegroundLinesHiddenLine: bool = proxy_property(bool,'ForegroundLinesHiddenLine',AccessMode.ReadWrite)
    ForegroundLinesLayer: str = proxy_property(str,'ForegroundLinesLayer',AccessMode.ReadWrite)
    ForegroundLinesLinetype: str = proxy_property(str,'ForegroundLinesLinetype',AccessMode.ReadWrite)
    ForegroundLinesLinetypeScale: float = proxy_property(float,'ForegroundLinesLinetypeScale',AccessMode.ReadWrite)
    ForegroundLinesLineweight: AcLineWeight = proxy_property('AcLineWeight','ForegroundLinesLineweight',AccessMode.ReadWrite)
    ForegroundLinesPlotStyleName: str = proxy_property(str,'ForegroundLinesPlotStyleName',AccessMode.ReadWrite)
    ForegroundLinesVisible: bool = proxy_property(bool,'ForegroundLinesVisible',AccessMode.ReadWrite)
    GenerationOptions: AcSectionGeneration = proxy_property('AcSectionGeneration','GenerationOptions',AccessMode.ReadWrite)
    IntersectionBoundaryColor: AcadAcCmColor = proxy_property('AcadAcCmColor','IntersectionBoundaryColor',AccessMode.ReadWrite)
    IntersectionBoundaryDivisionLines: bool = proxy_property(bool,'IntersectionBoundaryDivisionLines',AccessMode.ReadWrite)
    IntersectionBoundaryLayer: str = proxy_property(str,'IntersectionBoundaryLayer',AccessMode.ReadWrite)
    IntersectionBoundaryLinetype: str = proxy_property(str,'IntersectionBoundaryLinetype',AccessMode.ReadWrite)
    IntersectionBoundaryLinetypeScale: float = proxy_property(float,'IntersectionBoundaryLinetypeScale',AccessMode.ReadWrite)
    IntersectionBoundaryLineweight: AcLineWeight = proxy_property('AcLineWeight','IntersectionBoundaryLineweight',AccessMode.ReadWrite)
    IntersectionBoundaryPlotStyleName: str = proxy_property(str,'IntersectionBoundaryPlotStyleName',AccessMode.ReadWrite)
    IntersectionBoundaryVisible: bool = proxy_property(bool,'IntersectionBoundaryVisible',AccessMode.ReadWrite)
    IntersectionFillColor: AcadAcCmColor = proxy_property('AcadAcCmColor','IntersectionFillColor',AccessMode.ReadWrite)
    IntersectionFillFaceTransparency: int = proxy_property(int,'IntersectionFillFaceTransparency',AccessMode.ReadWrite)
    IntersectionFillHatchAngle: float = proxy_property(float,'IntersectionFillHatchAngle',AccessMode.ReadWrite)
    IntersectionFillHatchPatternName: str = proxy_property(str,'IntersectionFillHatchPatternName',AccessMode.ReadWrite)
    IntersectionFillHatchPatternType: AcPatternType = proxy_property(AcPatternType,'IntersectionFillHatchPatternType',AccessMode.ReadWrite)
    IntersectionFillHatchScale: float = proxy_property(float,'IntersectionFillHatchScale',AccessMode.ReadWrite)
    IntersectionFillHatchSpacing: float = proxy_property(float,'IntersectionFillHatchSpacing',AccessMode.ReadWrite)
    IntersectionFillLayer: str = proxy_property(str,'IntersectionFillLayer',AccessMode.ReadWrite)
    IntersectionFillLinetype: str = proxy_property(str,'IntersectionFillLinetype',AccessMode.ReadWrite)
    IntersectionFillLinetypeScale: float = proxy_property(float,'IntersectionFillLinetypeScale',AccessMode.ReadWrite)
    IntersectionFillLineweight: AcLineWeight = proxy_property('AcLineWeight','IntersectionFillLineweight',AccessMode.ReadWrite)
    IntersectionFillPlotStyleName: str = proxy_property(str,'IntersectionFillPlotStyleName',AccessMode.ReadWrite)
    IntersectionFillVisible: bool = proxy_property(bool,'IntersectionFillVisible',AccessMode.ReadWrite)
    SourceObjects: Variant = proxy_property('Variant','SourceObjects',AccessMode.ReadWrite)



# class PyGeTol:
#     """
#     Допуски для линейных и угловых измерений.
#     """
#     # Допуск по умолчанию для линейных/пространственных сравнений
#     # Стандартное значение 1e-9
#     kEps: float = 1e-9 
    
#     # Допуск для сравнения углов (в радианах)
#     kAngEps: float = 1e-12

#     # AcGeTol.kEqualPoint - допуск для точек
#     kEqualPoint: float = kEps
#     # AcGeTol.kEqualVector - допуск для векторов (для проверки на равенство)
#     kEqualVector: float = kEps
#     # AcGeTol.kZeroVector - допуск для проверки, является ли длина вектора нулевой
#     kZeroVector: float = 1e-10 

# geTol = PyGeTol()


from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *


    
class AcadPreferencesFiles(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    ActiveInvProject: str = proxy_property(str,'ActiveInvProject',AccessMode.ReadWrite)
    AltFontFile: str = proxy_property(str,'AltFontFile',AccessMode.ReadWrite)
    AltTabletMenuFile: str = proxy_property(str,'AltTabletMenuFile',AccessMode.ReadWrite)
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    AutoSavePath: str = proxy_property(str,'AutoSavePath',AccessMode.ReadWrite)
    ColorBookPath: str = proxy_property(str,'ColorBookPath',AccessMode.ReadWrite)
    ConfigFile: str = proxy_property(str,'ConfigFile',AccessMode.ReadOnly)
    CustomDictionary: str = proxy_property(str,'CustomDictionary',AccessMode.ReadWrite)
    CustomIconPath: str = proxy_property(str,'CustomIconPath',AccessMode.ReadWrite)
    DefaultInternetURL: str = proxy_property(str,'DefaultInternetURL',AccessMode.ReadWrite)
    DriversPath: str = proxy_property(str,'DriversPath',AccessMode.ReadWrite)
    EnterpriseMenuFile: str = proxy_property(str,'EnterpriseMenuFile',AccessMode.ReadWrite)
    FontFileMap: str = proxy_property(str,'FontFileMap',AccessMode.ReadWrite)
    HelpFilePath: str = proxy_property(str,'HelpFilePath',AccessMode.ReadWrite)
    LogFilePath: str = proxy_property(str,'LogFilePath',AccessMode.ReadWrite)
    MainDictionary: str = proxy_property(str,'MainDictionary',AccessMode.ReadWrite)
    MenuFile: str = proxy_property(str,'MenuFile',AccessMode.ReadWrite)
    PageSetupOverridesTemplateFile: str = proxy_property(str,'PageSetupOverridesTemplateFile',AccessMode.ReadWrite)
    PlotLogFilePath: str = proxy_property(str,'PlotLogFilePath',AccessMode.ReadWrite)
    PostScriptPrologFile: str = proxy_property(str,'PostScriptPrologFile',AccessMode.ReadWrite)
    PrinterConfigPath: str = proxy_property(str,'PrinterConfigPath',AccessMode.ReadWrite)
    PrinterDescPath: str = proxy_property(str,'PrinterDescPath',AccessMode.ReadWrite)
    PrinterStyleSheetPath: str = proxy_property(str,'PrinterStyleSheetPath',AccessMode.ReadWrite)
    PrintFile: str = proxy_property(str,'PrintFile',AccessMode.ReadWrite)
    PrintSpoolerPath: str = proxy_property(str,'PrintSpoolerPath',AccessMode.ReadWrite)
    PrintSpoolExecutable: str = proxy_property(str,'PrintSpoolExecutable',AccessMode.ReadWrite)
    QNewTemplateFile: str = proxy_property(str,'QNewTemplateFile',AccessMode.ReadWrite)
    SupportPath: str = proxy_property(str,'SupportPath',AccessMode.ReadWrite)
    TempFilePath: str = proxy_property(str,'TempFilePath',AccessMode.ReadWrite)
    TemplateDWGPath: str = proxy_property(str,'TemplateDWGPath',AccessMode.ReadWrite)
    TempXRefPath: str = proxy_property(str,'TempXRefPath',AccessMode.ReadWrite)
    TextEditor: str = proxy_property(str,'TextEditor',AccessMode.ReadWrite)
    TextureMapPath: str = proxy_property(str,'TextureMapPath',AccessMode.ReadWrite)
    ToolPalettePath: str = proxy_property(str,'ToolPalettePath',AccessMode.ReadWrite)
    WorkspacePath: str = proxy_property(str,'WorkspacePath',AccessMode.ReadWrite)

    def GetProjectFilePath(self, ProjectName: str) -> Out[str]:
        return self._obj.GetProjectFilePath(ProjectName)
        
    def SetProjectFilePath(self, ProjectName: str, ProjectFilePath: str) -> None:
        self._obj.SetProjectFilePath(ProjectName, ProjectFilePath)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPointCloudEx(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    ColorScheme: str = proxy_property(str,'ColorScheme',AccessMode.ReadWrite)
    Geolocate: bool = proxy_property(bool,'Geolocate',AccessMode.ReadWrite)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    Locked: bool = proxy_property(bool,'Locked',AccessMode.ReadWrite)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Path: str = proxy_property(str,'Path',AccessMode.ReadOnly)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    Scale: float = proxy_property(float,'Scale',AccessMode.ReadWrite)
    Segmentation: str = proxy_property(str,'Segmentation',AccessMode.ReadOnly)
    ShowCropped: bool = proxy_property(bool,'ShowCropped',AccessMode.ReadWrite)
    Stylization: AcPointCloudExStylizationType = proxy_property('AcPointCloudExStylizationType','Stylization',AccessMode.ReadWrite)
    Unit: str = proxy_property(str,'Unit',AccessMode.ReadOnly)
    UnitFactor: str = proxy_property(str,'UnitFactor',AccessMode.ReadOnly)

    def Copy(self) -> AcadPointCloudEx:
        return AcadPointCloudEx(self._obj.Copy())


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadRegion(AcadEntity):
    def __init__(self, obj) -> None: ...
    Area: float
    Centroid: PyGePoint2d
    MomentOfInertia: PyGePoint3d
    Normal: PyGeVector3d
    Perimeter: float
    PrincipalDirections: PyGeVector3d
    PrincipalMoments: PyGeVector3d
    ProductOfInertia: PyGeVector3d
    RadiiOfGyration: PyGeVector3d
    def Boolean(self, Operation: AcBooleanType, Object: Acad3DSolid | AcadRegion) -> None: ...
    def Copy(self) -> AcadRegion: ...
    def Explode(self) -> vObjectArray: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadRegion(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Area: float = proxy_property(float,'Area',AccessMode.ReadOnly)
    Centroid: PyGePoint2d = proxy_property('PyGePoint2d','Centroid',AccessMode.ReadOnly)
    MomentOfInertia: PyGePoint3d = proxy_property('PyGePoint3d','MomentOfInertia',AccessMode.ReadOnly)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Perimeter: float = proxy_property(float,'Normal',AccessMode.ReadOnly)
    PrincipalDirections: PyGeVector3d = proxy_property('PyGeVector3d','PrincipalDirections',AccessMode.ReadOnly)
    PrincipalMoments: PyGeVector3d = proxy_property('PyGeVector3d','PrincipalMoments',AccessMode.ReadOnly)
    ProductOfInertia: PyGeVector3d = proxy_property('PyGeVector3d','ProductOfInertia',AccessMode.ReadOnly)
    RadiiOfGyration: PyGeVector3d = proxy_property('PyGeVector3d','RadiiOfGyration',AccessMode.ReadOnly)

    def Boolean(self, Operation: AcBooleanType, Object: Acad3DSolid | AcadRegion ) -> None:
        self._obj.Boolean(Operation.value, Object())
        
    def Copy(self) -> AcadRegion:
        return AcadRegion(self._obj.Copy())
        
    def Explode(self) -> vObjectArray:
        return vObjectArray(self._obj.Explode())


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadPolyfaceMesh(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3dArray = proxy_property('PyGePoint3dArray','Coordinates',AccessMode.ReadWrite)
    NumberOfFaces: int = proxy_property(int,'NumberOfFaces',AccessMode.ReadOnly)
    NumberOfVertices: int = proxy_property(int,'NumberOfVertices',AccessMode.ReadOnly)

    def Coordinate(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.Coordinate(Index))

    def Copy(self) -> AcadPolyfaceMesh:
        return AcadPolyfaceMesh(self._obj.Copy())



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlock import IAcadBlock


class AcadPaperSpace(IAcadBlock):
    def __init__(self, obj) -> None: super().__init__(obj)

    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)

    def AddPViewport(self, Center: PyGePoint3d, Width: float, Height: float) -> AcadPViewport:
        return AcadPViewport(self._obj.AddPViewport(Center(), Width, Height))




from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadAcCmColor(AppObject):
    def __init__(self, obj) -> None: ...
    Blue: int
    BookName: str
    ColorIndex: AcColor
    ColorMethod: AcColorMethod
    ColorName: str
    EntityColor: int
    Green: int
    Red: int
    def Delete(self) -> None: ...
    def SetColorBookColor(self, BookName: str, ColorName: str) -> None: ...
    def SetNames(self, ColorName: str, ColorBook: str) -> None: ...
    def SetRGB(self, Red: int, Green: int, Blue: int) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPViewport(AcadEntity):
    def __init__(self, obj) -> None: ...
    ArcSmoothness: int
    Center: PyGePoint3d
    Clipped: bool
    CustomScale: float
    Direction: PyGeVector3d
    DisplayLocked: bool
    GridOn: bool
    HasSheetView: bool
    Height: float
    LabelBlockId: int
    LayerPropertyOverrides: bool
    LensLength: float
    ModelView: AcadView
    ShadePlot: AcShadePlot
    SheetView: AcadView
    SnapBasePoint: PyGePoint2d
    SnapOn: bool
    SnapRotationAngle: float
    StandardScale: AcViewportScale
    StandardScale2: int
    Target: PyGePoint3d
    TwistAngle: float
    UCSIconAtOrigin: bool
    UCSIconOn: bool
    UCSPerViewport: bool
    ViewportOn: bool
    VisualStyle: int
    Width: float
    def Copy(self) -> AcadPViewport: ...
    def GetGridSpacing(self) -> tuple: ...
    def GetSnapSpacing(self) -> tuple: ...
    def SetGridSpacing(self, XSpacing: float, YSpacing: float) -> None: ...
    def SetSnapSpacing(self, XSpacing: float, YSpacing: float) -> None: ...
    def SyncModelView(self) -> None: ...



from win32com.client import VARIANT
import pythoncom
import array
import numpy as np
from enum import IntEnum
from collections.abc import Iterable



def is_iterable(obj):
    return isinstance(obj, Iterable) and not isinstance(obj, (str, bytes, bytearray))

def flatten_generic(nested_iterable):
    flat_list = []
    if is_iterable(nested_iterable):
        for item in nested_iterable:
            if is_iterable(item):
                flat_list.extend(flatten_generic(item))
            else:
                flat_list.append(item)
    else:
        flat_list.append(nested_iterable)
    return flat_list

class VT(IntEnum):
    BOOL            = pythoncom.VT_BOOL
    FLOAT           = pythoncom.VT_R4
    DOUBLE          = pythoncom.VT_R8
    LONG            = pythoncom.VT_I8
    INT             = pythoncom.VT_I4
    SHORT           = pythoncom.VT_I2
    STRING          = pythoncom.VT_BSTR
    OBJECT          = pythoncom.VT_DISPATCH
    VARIANT         = pythoncom.VT_VARIANT
    EMPTY           = pythoncom.VT_EMPTY
    NULL            = pythoncom.VT_NULL
    ARRAY           = pythoncom.VT_ARRAY
    ARRAY_BOOL      = BOOL | ARRAY
    ARRAY_FLOAT     = FLOAT | ARRAY
    ARRAY_DOUBLE    = DOUBLE | ARRAY
    ARRAY_LONG      = LONG | ARRAY
    ARRAY_INT       = INT | ARRAY
    ARRAY_SHORT     = SHORT | ARRAY
    ARRAY_STRING    = STRING | ARRAY
    ARRAY_OBJECT    = OBJECT | ARRAY
    ARRAY_VARIANT   = VARIANT | ARRAY

class Variant():

    VT_TYPE_MAP = {
        int: VT.LONG, 
        float: VT.DOUBLE, 
        str: VT.STRING, 
        bool: VT.BOOL, 
        type(None): VT.EMPTY
    }

    def __init__(self, value, force_vt_type:VT = None):
        self._vt_type_initial:VT = None
        self._value_py:any = None
        self._force_iterable:bool = False
        if isinstance(value, Variant):
            self._value_py = value._value_py
            self._vt_type_initial = value._vt_type_initial
            return
        if isinstance(value, (array.array, np.ndarray)):
            if force_vt_type is None:
                raise ValueError("Необходимо указать тип значений массива")
            self._value_py = value
            self._vt_type_initial = force_vt_type
            return
        self._process_value(value, force_vt_type)

    @property
    def value(self):
        if self._vt_type_initial in (VT.NULL, VT.EMPTY):
            return None
        if self._force_iterable and not is_iterable(self._value_py):
            return [self._value_py]
        else:
            return self._value_py
    @property
    def variant(self): 
        return self.to_variant()

    @property
    def varianttype(self) -> VT:
        return VT(self._vt_type_initial)

    @property
    def valuetype(self) -> type:
        if self._force_iterable:
            t = type(self[0])
        else:
            self._force_iterable = True
            t = type(self[0])
            self._force_iterable = False
        return t

    @staticmethod
    def _convert_to_type(value, target_vt_type):
        if target_vt_type in (VT.SHORT, VT.INT, VT.LONG):
            try:
                return int(value)
            except (ValueError, TypeError):
                raise TypeError(f"Невозможно преобразовать '{value}' в целое число для типа VT_I.")
        elif target_vt_type in (VT.FLOAT, VT.DOUBLE):
            try:
                return float(value)
            except (ValueError, TypeError):
                raise TypeError(f"Невозможно преобразовать '{value}' в float для типа VT_R.")
        elif target_vt_type == VT.STRING:
            return str(value)
        elif target_vt_type == VT.BOOL:
            return bool(value)
        return value

    @staticmethod
    def _unpack_variant_recursive(value):
        if isinstance(value, VARIANT):
            unpacked_value = value.value
            return Variant._unpack_variant_recursive(unpacked_value)
        elif isinstance(value, (list, tuple)):
            return [Variant._unpack_variant_recursive(item) for item in value]
        else:
            return value

    def _process_value(self, value, force_vt_type=None):
        """Централизованная функция обработки значения, используется в __init__ и set_value."""
        if force_vt_type is not None:
            # Логика принудительного типа
            self._vt_type_initial = force_vt_type
            if force_vt_type & VT.ARRAY:
                data_list = value if isinstance(value, (list, tuple)) else [value]
                base_type_vt = force_vt_type ^ VT.ARRAY
                self._value_py = [self._convert_to_type(item, base_type_vt) for item in data_list]
            else:
                self._value_py = self._convert_to_type(value, force_vt_type)
        else:
            # Логика автоматического определения типа (если не было принудительного указания в __init__)
            # Если тип уже был установлен автоматически при инициализации, используем его для приведения
            if self._vt_type_initial is not None and not (self._vt_type_initial & VT.ARRAY):
                 self._value_py = self._convert_to_type(value, self._vt_type_initial)
            else:
                 # Иначе определяем тип заново (например, для массивов или первой инициализации)
                 self._value_py = value
                 self._vt_type_initial = self._determine_vt_type(value)

    def _determine_vt_type(self, value):
        """Определяет базовый тип VT_* и флаги VT_ARRAY на основе первого элемента массива."""
        if isinstance(value, VARIANT):
            self._value_py = self._unpack_variant_recursive(value)
            return value.varianttype

        value_type = type(value)

        # 1. Если это базовый тип Python
        if value_type in self.VT_TYPE_MAP:
            return self.VT_TYPE_MAP[value_type]

        # 2. Если это итерируемый объект (список, кортеж)
        if isinstance(value, (list, tuple)):
            self._value_py = self._unpack_variant_recursive(value) # Убедимся, что данные чистые
            
            if not value:
                # Для пустого списка используем VT_VARIANT по умолчанию
                base_vt = VT.VARIANT
            else:
                # Определяем тип по первому элементу
                # Рекурсивно вызываем _determine_vt_type для первого элемента 
                base_vt = self._determine_vt_type(value[0])
                
                # Убедимся, что базовый тип не является сам массивом (мы хотим только базовый тип здесь)
                if base_vt & VT.ARRAY:
                    # Если первый элемент массива сам является массивом, оставляем VT_VARIANT
                    base_vt = VT.VARIANT
                # (В идеале здесь должна быть проверка, что все элементы списка имеют тот же базовый тип)
            return VT.ARRAY | base_vt

        # 3. Если тип не поддерживается
        raise ValueError(f"Неподдерживаемый тип данных: {value_type}")

    def set_iterable(self, is_iterable:bool = True):
        self._force_iterable = is_iterable

    def __iter__(self):
            """Делает объект итерируемым, если внутреннее значение является списком/кортежем."""
            if isinstance(self._value_py, (list, tuple)):
                return iter(self._value_py)
            else:
                # Если это не массив, вызываем TypeError (как обычные неитерируемые объекты Python)
                if self._force_iterable:
                    return iter([self._value_py])
                else:
                    raise TypeError(f"Объект {self.__class__.__name__} с типом {type(self._value_py)} не итерируем.")

    def __getitem__(self, key):
        """Получение элемента или среза по индексу (obj[0] или obj[1:3])."""
        if isinstance(self._value_py, (list, tuple)):
            return self._value_py[key]
        else:
            if self._force_iterable:
                return [self._value_py][key]
            else:
                raise TypeError(f"Объект {self.__class__.__name__} с типом {type(self._value_py)} не поддерживает индексацию.")

    def __setitem__(self, key, value):
        """Установка элемента или среза по индексу (obj[0] = value)."""
        if isinstance(self._value_py, list): # Работаем только с изменяемыми списками
            # Определяем базовый VT_тип для приведения нового значения
            if not (self._vt_type_initial & VT.ARRAY):
                 raise TypeError("Невозможно установить элемент по индексу для не-массива.")
            base_vt = self._vt_type_initial ^ VT.ARRAY
            # Если пользователь передал срез (slice), value будет списком
            if isinstance(key, slice):
                # Приводим каждый элемент нового списка к целевому типу
                converted_values = [self._convert_to_type(item, base_vt) for item in value]
                self._value_py[key] = converted_values
            else:
                # Если передан один индекс, приводим одно значение
                converted_value = self._convert_to_type(value, base_vt)
                self._value_py[key] = converted_value
        else:
            raise TypeError(f"Объект {self.__class__.__name__} с типом {type(self._value_py)} не поддерживает присваивание по индексу.")
            
    def __len__(self):
        """Возвращает длину объекта, если он является массивом."""
        if isinstance(self._value_py, (list, tuple)):
            return len(self._value_py)
        else:
            if self._force_iterable: return 1
            return 0 # Или вызвать TypeError, в зависимости от желаемого поведения

    def to_variant(self, force_vt_type:VT = None):
        target_vt = force_vt_type if force_vt_type is not None else self._vt_type_initial
        return VARIANT(target_vt, self._value_py)

    def __call__(self, force_vt_type=None): return self.to_variant(force_vt_type)

    def __str__(self):
        return f"{self.__class__.__name__}({self.varianttype!r}, {self.value})"

    def __repr__(self):
        return self.variant


vObjectEmpty = Variant(pythoncom.Empty, VT.OBJECT | VT.EMPTY)


class vObject(Variant):
    def __init__(self, obj):
        super().__init__(obj, force_vt_type=VT.OBJECT)

class vDouble(Variant):
    def __init__(self, double):
        super().__init__(double, force_vt_type=VT.DOUBLE)

class vLong(Variant):
    def __init__(self, long):
        super().__init__(long, force_vt_type=VT.LONG)

class vInteger(Variant):
    def __init__(self, integer):
        super().__init__(integer, force_vt_type=VT.INT)

class vShort(Variant):
    def __init__(self, boolean):
        super().__init__(boolean, force_vt_type=VT.SHORT)

class vBool(Variant):
    def __init__(self, boolean):
        super().__init__(boolean, force_vt_type=VT.BOOL)

class vString(Variant):
    def __init__(self, string):
        super().__init__(string, force_vt_type=VT.STRING)


class vArray(Variant):
    def __init__(self, *args, data_type:VT, check_count, set_default):
        arglist = flatten_generic(list(args))
        if check_count is not None:
            if set_default is not None:
                while len(arglist) < check_count:
                    arglist.append(set_default)
            if len(arglist) != check_count: 
                raise ValueError(f'Некорректное количество параметров: {len(arglist)}, ограничение: {check_count}')
        super().__init__(arglist, force_vt_type = VT.ARRAY | data_type)

class vObjectArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(args, data_type = VT.OBJECT, check_count = check_count, set_default = set_default)

class vVariantArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.VARIANT, check_count = check_count, set_default = set_default)

class vDoubleArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.DOUBLE, check_count = check_count, set_default = set_default)

class vLongArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.LONG, check_count = check_count, set_default = set_default)

class vStringArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.STRING, check_count = check_count, set_default = set_default)

class vIntegerArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.INT, check_count = check_count, set_default = set_default)

class vBoolArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.BOOL, check_count = check_count, set_default = set_default)

class vShortArray(vArray):
    def __init__(self, *args, check_count = None, set_default = None):
        super().__init__(*args, data_type = VT.SHORT, check_count = check_count, set_default = set_default)



from ..Base import *
from ..Proxy import *
from .AcadSurface import AcadSurface as AcadSurface

class AcadExtrudedSurface(AcadSurface):
    def __init__(self, obj) -> None: ...
    Direction: PyGeVector3d
    Height: float
    TaperAngle: float
    def Copy(self) -> AcadExtrudedSurface: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadBlockReference(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    EffectiveName: str = proxy_property(str,'EffectiveName',AccessMode.ReadOnly)
    HasAttributes: bool = proxy_property(bool,'EffectiveName',AccessMode.ReadOnly)
    InsertionPoint: PyGePoint3d = proxy_property('PyGePoint3d','InsertionPoint',AccessMode.ReadWrite)
    InsUnits: str = proxy_property(str,'InsUnits',AccessMode.ReadOnly)
    InsUnitsFactor: float = proxy_property(float,'InsUnitsFactor',AccessMode.ReadOnly)
    IsDynamicBlock: bool = proxy_property(bool,'IsDynamicBlock',AccessMode.ReadOnly)
    Name: bool = proxy_property(bool,'Name',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    XEffectiveScaleFactor: float = proxy_property(float,'XEffectiveScaleFactor',AccessMode.ReadWrite)
    XScaleFactor: float = proxy_property(float,'XScaleFactor',AccessMode.ReadWrite)
    YEffectiveScaleFactor: float = proxy_property(float,'YEffectiveScaleFactor',AccessMode.ReadWrite)
    YScaleFactor: float = proxy_property(float,'YScaleFactor',AccessMode.ReadWrite)
    ZEffectiveScaleFactor: float = proxy_property(float,'ZEffectiveScaleFactor',AccessMode.ReadWrite)
    ZScaleFactor: float = proxy_property(float,'ZEffectiveScaleFactor',AccessMode.ReadWrite)

    def ConvertToAnonymousBlock(self) -> None:
        self._obj.ConvertToAnonymousBlock()

    def ConvertToStaticBlock(self, newBlockName: str) -> None:
        self._obj.ConvertToStaticBlock(newBlockName)

    def Copy(self) -> AcadBlockReference:
        return AcadBlockReference(self._obj.Copy())
        
    def Explode(self) -> vObjectArray:
        return vObjectArray(self._obj.Explode())
        
    def GetAttributes(self) -> vObjectArray[AcadAttributeReference]:
        return vObjectArray(self._obj.GetAttributes())
            
    def GetConstantAttributes(self) -> vObjectArray[AcadAttribute]:
        return vObjectArray(self._obj.GetConstantAttributes())
        
    def GetDynamicBlockProperties(self) -> vObjectArray[AcadDynamicBlockReferenceProperty]:
        return vObjectArray(self._obj.GetDynamicBlockProperties())
        
    def ResetBlock(self) -> None:
        self._obj.ResetBlock()


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadPointCloudEx(AcadEntity):
    def __init__(self, obj) -> None: ...
    ColorScheme: str
    Geolocate: bool
    InsertionPoint: PyGePoint3d
    Locked: bool
    Name: str
    Path: str
    Rotation: float
    Scale: float
    Segmentation: str
    ShowCropped: bool
    Stylization: AcPointCloudExStylizationType
    Unit: str
    UnitFactor: str
    def Copy(self) -> AcadPointCloudEx: ...



from ..Base import *
from ..Proxy import *
from .AcadDatabase import AcadDatabase as AcadDatabase, IAcadDatabase as IAcadDatabase
from .AcadUCS import AcadUCS as AcadUCS

class AcadDocument(AcadDatabase):
    def __init__(self, obj) -> None: ...
    Active: bool
    ActiveDimStyle: AcadDimStyle
    ActiveLayer: AcadLayer
    ActiveLayout: AcadLayout
    ActiveLinetype: AcadLineType
    ActiveMaterial: AcadMaterial
    ActivePViewport: AcadPViewport
    ActiveSelectionSet: AcadSelectionSet
    ActiveSpace: AcActiveSpace
    ActiveTextStyle: AcadTextStyle
    ActiveUCS: AcadUCS
    ActiveViewport: AcadViewport
    Application: AcadApplication
    Database: IAcadDatabase
    FullName: str
    Height: float
    HWND: int
    MSpace: bool
    Name: str
    ObjectSnapMode: bool
    Path: str
    PickfirstSelectionSet: AcadSelectionSet
    Plot: AcadPlot
    ReadOnly: bool
    Saved: bool
    SelectionSets: AcadSelectionSet
    SummaryInfo: AcadSummaryInfo
    Utility: AcadUtility
    Width: float
    WindowState: AcWindowState
    WindowTitle: str
    def Activate(self) -> None: ...
    def AuditInfo(self, FixError: bool) -> None: ...
    def Close(self, SaveChanges: bool = False, FileName: str = '') -> None: ...
    def EndUndoMark(self) -> None: ...
    def Export(self, FileName: str, Extension: str, SelectionSet: AcadSelectionSet) -> None: ...
    def GetVariable(self, Name: str) -> any: ...
    def Import(self, FileName: str, InsertionPoint: PyGePoint3d, ScaleFactor: float) -> AcadBlockReference | None: ...
    def LoadShapeFile(self, FullName: str) -> None: ...
    def New(self, TemplateFileName: str) -> AcadDocument: ...
    def Open(self, Name: str, ReadOnly: bool = False) -> AcadDocument: ...
    def PostCommand(self, Command: str) -> None: ...
    def PurgeAll(self) -> None: ...
    def Regen(self, WhichViewports: AcRegenType) -> None: ...
    def Save(self) -> None: ...
    def SaveAs(self, FileName: str, FileType: AcSaveAsType = ..., SecurityParams: AcadSecurityParams = None) -> None: ...
    def SendCommand(self, Command: str) -> None: ...
    def SetVariable(self, Name: str, Value: Variant) -> None: ...
    def StartUndoMark(self) -> None: ...
    def WBlock(self, FileName: str, SelectionSet: AcadSelectionSet) -> None: ...



from __future__ import annotations
from enum import IntEnum

class AccessMode(IntEnum):
    ReadWrite = 0
    ReadOnly = 1
    WriteOnly = 2
    DenyFromAll = 3

class proxy_property(): #Generic[T]
    def __init__(self, rettype: type, propertyName: str, mode: AccessMode):
        # Храним как строку ИЛИ как класс
        self.rettype_name = rettype if isinstance(rettype, str) else None
        self.rettype = rettype
        self.propertyName = propertyName
        self.mode = mode

    def __get__(self, instance, owner):
        if self.mode.value is AccessMode.WriteOnly.value:
            raise Exception(f"Свойство '{self.propertyName}' доступно только для записи.")
        if instance is None:
            return self
        # Если self.rettype_name - строка, динамически ищем класс
        target_type = None
        if self.rettype_name:
            try:
                # Ищем класс в глобальной области видимости или области видимости владельца (owner)
                target_type = globals().get(self.rettype_name) or getattr(owner, self.rettype_name, None)
                if not target_type:
                    raise NameError(f"Тип '{self.rettype_name}' не найден.")
                    # target_type = globals().get(self.rettype, None)
            except Exception as e: 
                # raise TypeError(f"Не удалось разрешить имя типа '{self.rettype_name}': {e}")
                pass
        else:
            target_type = self.rettype
        value = getattr(instance._obj, self.propertyName)
        if not target_type:
            return value
        else:
            return target_type(value)

    def __set__(self, instance, value):
        if self.mode is AccessMode.ReadOnly:
            raise AttributeError(f"Свойство '{self.propertyName}' доступно только для чтения.")
        # Логика установки значения в базовый объект
        try:
            if type(value).__mro__[-2] in (AppObject, Variant): value = value()
            setattr(instance._obj, self.propertyName, value)
        except AttributeError:
            raise AttributeError(f"Невозможно установить свойство '{self.propertyName}' в базовом объекте.")


from ..Types import *
from .Base import *

from .Objects.AcadAcCmColor import AcadAcCmColor
from .Objects.AcadApplication import AcadApplication
from .Objects.AcadBlock import AcadBlock
from .Objects.AcadBlocks import AcadBlocks
from .Objects.AcadDatabase import AcadDatabase, IAcadDatabase
from .Objects.AcadDatabasePreferences import AcadDatabasePreferences
from .Objects.AcadDictionaries import AcadDictionaries
from .Objects.AcadDictionary import AcadDictionary
from .Objects.AcadDimStyle import AcadDimStyle
from .Objects.AcadDimStyles import AcadDimStyles
from .Objects.AcadDocument import AcadDocument
from .Objects.AcadDocuments import AcadDocuments
from .Objects.AcadDynamicBlockReferenceProperty import AcadDynamicBlockReferenceProperty
from .Objects.AcadGroup import AcadGroup
from .Objects.AcadGroups import AcadGroups
from .Objects.AcadHyperlink import AcadHyperlink
from .Objects.AcadHyperlinks import AcadHyperlinks
from .Objects.AcadIDPair import AcadIDPair
from .Objects.AcadLayer import AcadLayer
from .Objects.AcadLayers import AcadLayers
from .Objects.AcadLayerStateManager import AcadLayerStateManager
from .Objects.AcadLayout import AcadLayout
from .Objects.AcadLayouts import AcadLayouts
from .Objects.AcadLineType import AcadLineType
from .Objects.AcadLineTypes import AcadLineTypes
from .Objects.AcadMaterial import AcadMaterial
from .Objects.AcadMaterials import AcadMaterials
from .Objects.AcadMenuBar import AcadMenuBar
from .Objects.AcadMenuGroup import AcadMenuGroup
from .Objects.AcadMenuGroups import AcadMenuGroups
from .Objects.AcadMLeaderLeader import AcadMLeaderLeader
from .Objects.AcadMLeaderStyle import AcadMLeaderStyle
from .Objects.AcadModelSpace import AcadModelSpace
from .Objects.AcadPaperSpace import AcadPaperSpace
from .Objects.AcadPlot import AcadPlot
from .Objects.AcadPlotConfiguration import AcadPlotConfiguration
from .Objects.AcadPlotConfigurations import AcadPlotConfigurations
from .Objects.AcadPopupMenu import AcadPopupMenu
from .Objects.AcadPopupMenuItem import AcadPopupMenuItem
from .Objects.AcadPopupMenus import AcadPopupMenus
from .Objects.AcadPreferences import AcadPreferences
from .Objects.AcadPreferencesDisplay import AcadPreferencesDisplay
from .Objects.AcadPreferencesDrafting import AcadPreferencesDrafting
from .Objects.AcadPreferencesFiles import AcadPreferencesFiles
from .Objects.AcadPreferencesOpenSave import AcadPreferencesOpenSave
from .Objects.AcadPreferencesOutput import AcadPreferencesOutput
from .Objects.AcadPreferencesProfiles import AcadPreferencesProfiles
from .Objects.AcadPreferencesSelection import AcadPreferencesSelection
from .Objects.AcadPreferencesSystem import AcadPreferencesSystem
from .Objects.AcadPreferencesUser import AcadPreferencesUser
from .Objects.AcadRegisteredApplication import AcadRegisteredApplication
from .Objects.AcadRegisteredApplications import AcadRegisteredApplications
from .Objects.AcadSectionManager import AcadSectionManager
from .Objects.AcadSectionSettings import AcadSectionSettings
from .Objects.AcadSectionTypeSettings import AcadSectionTypeSettings
from .Objects.AcadSecurityParams import AcadSecurityParams
from .Objects.AcadSelectionSet import AcadSelectionSet
from .Objects.AcadSelectionSets import AcadSelectionSets
from .Objects.AcadSortentsTable import AcadSortentsTable
from .Objects.AcadState import AcadState
from .Objects.AcadSubDMeshEdge import AcadSubDMeshEdge
from .Objects.AcadSubDMeshFace import AcadSubDMeshFace
from .Objects.AcadSubDMeshVertex import AcadSubDMeshVertex
from .Objects.AcadSubEntSolidEdge import AcadSubEntSolidEdge
from .Objects.AcadSubEntSolidFace import AcadSubEntSolidFace
from .Objects.AcadSubEntSolidNode import AcadSubEntSolidNode
from .Objects.AcadSubEntSolidVertex import AcadSubEntSolidVertex
from .Objects.AcadSummaryInfo import AcadSummaryInfo
from .Objects.AcadTableStyle import AcadTableStyle
from .Objects.AcadSubEntity import AcadSubEntity
from .Objects.AcadTextStyle import AcadTextStyle
from .Objects.AcadTextStyles import AcadTextStyles
from .Objects.AcadToolbar import AcadToolbar
from .Objects.AcadToolbarItem import AcadToolbarItem
from .Objects.AcadToolbars import AcadToolbars
from .Objects.AcadUCS import AcadUCS
from .Objects.AcadUCSs import AcadUCSs
from .Objects.AcadUtility import AcadUtility
from .Objects.AcadView import AcadView
from .Objects.AcadViewport import AcadViewport
from .Objects.AcadViewports import AcadViewports
from .Objects.AcadViews import AcadViews
from .Objects.AcadXRecord import AcadXRecord


from .Entities.Acad3DFace import Acad3DFace
from .Entities.Acad3DPolyline import Acad3DPolyline
from .Entities.Acad3DSolid import Acad3DSolid
from .Entities.AcadArc import AcadArc
from .Entities.AcadAttribute import AcadAttribute
from .Entities.AcadAttributeReference import AcadAttributeReference
from .Entities.AcadBlockReference import AcadBlockReference
from .Entities.AcadCircle import AcadCircle
from .Entities.AcadComparedReference import AcadComparedReference
from .Entities.AcadDgnUnderlay import AcadDgnUnderlay
from .Entities.AcadDim3PointAngular import AcadDim3PointAngular
from .Entities.AcadDimAligned import AcadDimAligned
from .Entities.AcadDimAngular import AcadDimAngular
from .Entities.AcadDimArcLength import AcadDimArcLength
from .Entities.AcadDimDiametric import AcadDimDiametric
from .Entities.AcadDimension import AcadDimension
from .Entities.AcadDimOrdinate import AcadDimOrdinate
from .Entities.AcadDimRadial import AcadDimRadial
from .Entities.AcadDimRadialLarge import AcadDimRadialLarge
from .Entities.AcadDimRotated import AcadDimRotated
from .Entities.AcadDwfUnderlay import AcadDwfUnderlay
from .Entities.AcadEllipse import AcadEllipse
from .Entities.AcadExternalReference import AcadExternalReference
from .Entities.AcadExtrudedSurface import AcadExtrudedSurface
from .Entities.AcadGeomapImage import AcadGeomapImage
from .Entities.AcadGeoPositionMarker import AcadGeoPositionMarker
from .Entities.AcadHatch import AcadHatch
from .Entities.AcadHelix import AcadHelix
from .Entities.AcadLeader import AcadLeader
from .Entities.AcadLine import AcadLine
from .Entities.AcadLoftedSurface import AcadLoftedSurface
from .Entities.AcadLWPolyline import AcadLWPolyline
from .Entities.AcadMInsertBlock import AcadMInsertBlock
from .Entities.AcadMLeader import AcadMLeader
from .Entities.AcadMLine import AcadMLine
from .Entities.AcadMtext import AcadMtext
from .Entities.AcadNurbSurface import AcadNurbSurface
from .Entities.AcadOle import AcadOle
from .Entities.AcadPdfUnderlay import AcadPdfUnderlay
from .Entities.AcadPlaneSurface import AcadPlaneSurface
from .Entities.AcadPoint import AcadPoint
from .Entities.AcadPointCloud import AcadPointCloud
from .Entities.AcadPointCloudEx import AcadPointCloudEx
from .Entities.AcadPolyfaceMesh import AcadPolyfaceMesh
from .Entities.AcadPolygonMesh import AcadPolygonMesh
from .Entities.AcadPolyline import AcadPolyline
from .Entities.AcadPViewport import AcadPViewport
from .Entities.AcadRasterImage import AcadRasterImage
from .Entities.AcadRay import AcadRay
from .Entities.AcadRegion import AcadRegion
from .Entities.AcadSection import AcadSection
from .Entities.AcadShape import AcadShape
from .Entities.AcadSolid import AcadSolid
from .Entities.AcadSpline import AcadSpline
from .Entities.AcadSubDMesh import AcadSubDMesh
from .Entities.AcadSurface import AcadSurface
from .Entities.AcadSweptSurface import AcadSweptSurface
from .Entities.AcadTable import AcadTable
from .Entities.AcadText import AcadText
from .Entities.AcadTolerance import AcadTolerance
from .Entities.AcadTrace import AcadTrace
from .Entities.AcadUnderlay import AcadUnderlay
from .Entities.AcadWipeout import AcadWipeout
from .Entities.AcadXline import AcadXline



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class Acad3DFace(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Coordinates: PyGePoint3dArray = proxy_property('PyGePoint3dArray','Coordinates',AccessMode.ReadWrite)
    VisibilityEdge1: bool = proxy_property(bool,'VisibilityEdge1',AccessMode.ReadWrite)
    VisibilityEdge2: bool = proxy_property(bool,'VisibilityEdge2',AccessMode.ReadWrite)
    VisibilityEdge3: bool = proxy_property(bool,'VisibilityEdge3',AccessMode.ReadWrite)
    VisibilityEdge4: bool = proxy_property(bool,'VisibilityEdge4',AccessMode.ReadWrite)
    

    def Coordinate(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.Coordinate(Index))

    def Copy(self) -> Acad3DFace:
        return Acad3DFace(self._obj.Copy())
        
    def GetInvisibleEdge(self, Index: int) -> bool:
        return self._obj.GetInvisibleEdge(Index)
        
    def SetInvisibleEdge(self, Index: int, State: bool) -> None:
        self._obj.SetInvisibleEdge(Index, State)


from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadGeoPositionMarker(AcadEntity):
    def __init__(self, obj) -> None: ...
    Altitude: float
    BackgroundFill: bool
    DrawingDirection: AcDrawingDirection
    Height: float
    LandingGap: float
    Latitude: str
    LineSpacingDistance: float
    LineSpacingFactor: float
    LineSpacingStyle: AcLineSpacingStyle
    Longitude: str
    Notes: str
    Position: PyGePoint3d
    Radius: float
    Rotation: float
    TextFrameDisplay: bool
    TextJustify: AcAttachmentPoint
    TextString: str
    TextStyleName: str
    TextWidth: float
    def Copy(self) -> AcadGeoPositionMarker: ...



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesSystem(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    BeepOnError: bool
    DisplayOLEScale: bool
    EnableStartupDialog: bool
    LoadAcadLspInAllDocuments: bool
    ShowWarningMessages: bool
    SingleDocumentMode: bool
    StoreSQLIndex: bool
    TablesReadOnly: bool



from ..Types import *
from .Base import *
from .Entities.Acad3DFace import Acad3DFace as Acad3DFace
from .Entities.Acad3DPolyline import Acad3DPolyline as Acad3DPolyline
from .Entities.Acad3DSolid import Acad3DSolid as Acad3DSolid
from .Entities.AcadArc import AcadArc as AcadArc
from .Entities.AcadAttribute import AcadAttribute as AcadAttribute
from .Entities.AcadAttributeReference import AcadAttributeReference as AcadAttributeReference
from .Entities.AcadBlockReference import AcadBlockReference as AcadBlockReference
from .Entities.AcadCircle import AcadCircle as AcadCircle
from .Entities.AcadComparedReference import AcadComparedReference as AcadComparedReference
from .Entities.AcadDgnUnderlay import AcadDgnUnderlay as AcadDgnUnderlay
from .Entities.AcadDim3PointAngular import AcadDim3PointAngular as AcadDim3PointAngular
from .Entities.AcadDimAligned import AcadDimAligned as AcadDimAligned
from .Entities.AcadDimAngular import AcadDimAngular as AcadDimAngular
from .Entities.AcadDimArcLength import AcadDimArcLength as AcadDimArcLength
from .Entities.AcadDimDiametric import AcadDimDiametric as AcadDimDiametric
from .Entities.AcadDimOrdinate import AcadDimOrdinate as AcadDimOrdinate
from .Entities.AcadDimRadial import AcadDimRadial as AcadDimRadial
from .Entities.AcadDimRadialLarge import AcadDimRadialLarge as AcadDimRadialLarge
from .Entities.AcadDimRotated import AcadDimRotated as AcadDimRotated
from .Entities.AcadDimension import AcadDimension as AcadDimension
from .Entities.AcadDwfUnderlay import AcadDwfUnderlay as AcadDwfUnderlay
from .Entities.AcadEllipse import AcadEllipse as AcadEllipse
from .Entities.AcadExternalReference import AcadExternalReference as AcadExternalReference
from .Entities.AcadExtrudedSurface import AcadExtrudedSurface as AcadExtrudedSurface
from .Entities.AcadGeoPositionMarker import AcadGeoPositionMarker as AcadGeoPositionMarker
from .Entities.AcadGeomapImage import AcadGeomapImage as AcadGeomapImage
from .Entities.AcadHatch import AcadHatch as AcadHatch
from .Entities.AcadHelix import AcadHelix as AcadHelix
from .Entities.AcadLWPolyline import AcadLWPolyline as AcadLWPolyline
from .Entities.AcadLeader import AcadLeader as AcadLeader
from .Entities.AcadLine import AcadLine as AcadLine
from .Entities.AcadLoftedSurface import AcadLoftedSurface as AcadLoftedSurface
from .Entities.AcadMInsertBlock import AcadMInsertBlock as AcadMInsertBlock
from .Entities.AcadMLeader import AcadMLeader as AcadMLeader
from .Entities.AcadMLine import AcadMLine as AcadMLine
from .Entities.AcadMtext import AcadMtext as AcadMtext
from .Entities.AcadNurbSurface import AcadNurbSurface as AcadNurbSurface
from .Entities.AcadOle import AcadOle as AcadOle
from .Entities.AcadPViewport import AcadPViewport as AcadPViewport
from .Entities.AcadPdfUnderlay import AcadPdfUnderlay as AcadPdfUnderlay
from .Entities.AcadPlaneSurface import AcadPlaneSurface as AcadPlaneSurface
from .Entities.AcadPoint import AcadPoint as AcadPoint
from .Entities.AcadPointCloud import AcadPointCloud as AcadPointCloud
from .Entities.AcadPointCloudEx import AcadPointCloudEx as AcadPointCloudEx
from .Entities.AcadPolyfaceMesh import AcadPolyfaceMesh as AcadPolyfaceMesh
from .Entities.AcadPolygonMesh import AcadPolygonMesh as AcadPolygonMesh
from .Entities.AcadPolyline import AcadPolyline as AcadPolyline
from .Entities.AcadRasterImage import AcadRasterImage as AcadRasterImage
from .Entities.AcadRay import AcadRay as AcadRay
from .Entities.AcadRegion import AcadRegion as AcadRegion
from .Entities.AcadSection import AcadSection as AcadSection
from .Entities.AcadShape import AcadShape as AcadShape
from .Entities.AcadSolid import AcadSolid as AcadSolid
from .Entities.AcadSpline import AcadSpline as AcadSpline
from .Entities.AcadSubDMesh import AcadSubDMesh as AcadSubDMesh
from .Entities.AcadSurface import AcadSurface as AcadSurface
from .Entities.AcadSweptSurface import AcadSweptSurface as AcadSweptSurface
from .Entities.AcadTable import AcadTable as AcadTable
from .Entities.AcadText import AcadText as AcadText
from .Entities.AcadTolerance import AcadTolerance as AcadTolerance
from .Entities.AcadTrace import AcadTrace as AcadTrace
from .Entities.AcadUnderlay import AcadUnderlay as AcadUnderlay
from .Entities.AcadWipeout import AcadWipeout as AcadWipeout
from .Entities.AcadXline import AcadXline as AcadXline
from .Objects.AcadAcCmColor import AcadAcCmColor as AcadAcCmColor
from .Objects.AcadApplication import AcadApplication as AcadApplication
from .Objects.AcadBlock import AcadBlock as AcadBlock
from .Objects.AcadBlocks import AcadBlocks as AcadBlocks
from .Objects.AcadDatabase import AcadDatabase as AcadDatabase, IAcadDatabase as IAcadDatabase
from .Objects.AcadDatabasePreferences import AcadDatabasePreferences as AcadDatabasePreferences
from .Objects.AcadDictionaries import AcadDictionaries as AcadDictionaries
from .Objects.AcadDictionary import AcadDictionary as AcadDictionary
from .Objects.AcadDimStyle import AcadDimStyle as AcadDimStyle
from .Objects.AcadDimStyles import AcadDimStyles as AcadDimStyles
from .Objects.AcadDocument import AcadDocument as AcadDocument
from .Objects.AcadDocuments import AcadDocuments as AcadDocuments
from .Objects.AcadDynamicBlockReferenceProperty import AcadDynamicBlockReferenceProperty as AcadDynamicBlockReferenceProperty
from .Objects.AcadGroup import AcadGroup as AcadGroup
from .Objects.AcadGroups import AcadGroups as AcadGroups
from .Objects.AcadHyperlink import AcadHyperlink as AcadHyperlink
from .Objects.AcadHyperlinks import AcadHyperlinks as AcadHyperlinks
from .Objects.AcadIDPair import AcadIDPair as AcadIDPair
from .Objects.AcadLayer import AcadLayer as AcadLayer
from .Objects.AcadLayerStateManager import AcadLayerStateManager as AcadLayerStateManager
from .Objects.AcadLayers import AcadLayers as AcadLayers
from .Objects.AcadLayout import AcadLayout as AcadLayout
from .Objects.AcadLayouts import AcadLayouts as AcadLayouts
from .Objects.AcadLineType import AcadLineType as AcadLineType
from .Objects.AcadLineTypes import AcadLineTypes as AcadLineTypes
from .Objects.AcadMLeaderLeader import AcadMLeaderLeader as AcadMLeaderLeader
from .Objects.AcadMLeaderStyle import AcadMLeaderStyle as AcadMLeaderStyle
from .Objects.AcadMaterial import AcadMaterial as AcadMaterial
from .Objects.AcadMaterials import AcadMaterials as AcadMaterials
from .Objects.AcadMenuBar import AcadMenuBar as AcadMenuBar
from .Objects.AcadMenuGroup import AcadMenuGroup as AcadMenuGroup
from .Objects.AcadMenuGroups import AcadMenuGroups as AcadMenuGroups
from .Objects.AcadModelSpace import AcadModelSpace as AcadModelSpace
from .Objects.AcadPaperSpace import AcadPaperSpace as AcadPaperSpace
from .Objects.AcadPlot import AcadPlot as AcadPlot
from .Objects.AcadPlotConfiguration import AcadPlotConfiguration as AcadPlotConfiguration
from .Objects.AcadPlotConfigurations import AcadPlotConfigurations as AcadPlotConfigurations
from .Objects.AcadPopupMenu import AcadPopupMenu as AcadPopupMenu
from .Objects.AcadPopupMenuItem import AcadPopupMenuItem as AcadPopupMenuItem
from .Objects.AcadPopupMenus import AcadPopupMenus as AcadPopupMenus
from .Objects.AcadPreferences import AcadPreferences as AcadPreferences
from .Objects.AcadPreferencesDisplay import AcadPreferencesDisplay as AcadPreferencesDisplay
from .Objects.AcadPreferencesDrafting import AcadPreferencesDrafting as AcadPreferencesDrafting
from .Objects.AcadPreferencesFiles import AcadPreferencesFiles as AcadPreferencesFiles
from .Objects.AcadPreferencesOpenSave import AcadPreferencesOpenSave as AcadPreferencesOpenSave
from .Objects.AcadPreferencesOutput import AcadPreferencesOutput as AcadPreferencesOutput
from .Objects.AcadPreferencesProfiles import AcadPreferencesProfiles as AcadPreferencesProfiles
from .Objects.AcadPreferencesSelection import AcadPreferencesSelection as AcadPreferencesSelection
from .Objects.AcadPreferencesSystem import AcadPreferencesSystem as AcadPreferencesSystem
from .Objects.AcadPreferencesUser import AcadPreferencesUser as AcadPreferencesUser
from .Objects.AcadRegisteredApplication import AcadRegisteredApplication as AcadRegisteredApplication
from .Objects.AcadRegisteredApplications import AcadRegisteredApplications as AcadRegisteredApplications
from .Objects.AcadSectionManager import AcadSectionManager as AcadSectionManager
from .Objects.AcadSectionSettings import AcadSectionSettings as AcadSectionSettings
from .Objects.AcadSectionTypeSettings import AcadSectionTypeSettings as AcadSectionTypeSettings
from .Objects.AcadSecurityParams import AcadSecurityParams as AcadSecurityParams
from .Objects.AcadSelectionSet import AcadSelectionSet as AcadSelectionSet
from .Objects.AcadSelectionSets import AcadSelectionSets as AcadSelectionSets
from .Objects.AcadSortentsTable import AcadSortentsTable as AcadSortentsTable
from .Objects.AcadState import AcadState as AcadState
from .Objects.AcadSubDMeshEdge import AcadSubDMeshEdge as AcadSubDMeshEdge
from .Objects.AcadSubDMeshFace import AcadSubDMeshFace as AcadSubDMeshFace
from .Objects.AcadSubDMeshVertex import AcadSubDMeshVertex as AcadSubDMeshVertex
from .Objects.AcadSubEntSolidEdge import AcadSubEntSolidEdge as AcadSubEntSolidEdge
from .Objects.AcadSubEntSolidFace import AcadSubEntSolidFace as AcadSubEntSolidFace
from .Objects.AcadSubEntSolidNode import AcadSubEntSolidNode as AcadSubEntSolidNode
from .Objects.AcadSubEntSolidVertex import AcadSubEntSolidVertex as AcadSubEntSolidVertex
from .Objects.AcadSubEntity import AcadSubEntity as AcadSubEntity
from .Objects.AcadSummaryInfo import AcadSummaryInfo as AcadSummaryInfo
from .Objects.AcadTableStyle import AcadTableStyle as AcadTableStyle
from .Objects.AcadTextStyle import AcadTextStyle as AcadTextStyle
from .Objects.AcadTextStyles import AcadTextStyles as AcadTextStyles
from .Objects.AcadToolbar import AcadToolbar as AcadToolbar
from .Objects.AcadToolbarItem import AcadToolbarItem as AcadToolbarItem
from .Objects.AcadToolbars import AcadToolbars as AcadToolbars
from .Objects.AcadUCS import AcadUCS as AcadUCS
from .Objects.AcadUCSs import AcadUCSs as AcadUCSs
from .Objects.AcadUtility import AcadUtility as AcadUtility
from .Objects.AcadView import AcadView as AcadView
from .Objects.AcadViewport import AcadViewport as AcadViewport
from .Objects.AcadViewports import AcadViewports as AcadViewports
from .Objects.AcadViews import AcadViews as AcadViews
from .Objects.AcadXRecord import AcadXRecord as AcadXRecord
from enum import IntEnum

class AccessMode(IntEnum):
    ReadWrite = 0
    ReadOnly = 1
    WriteOnly = 2
    DenyFromAll = 3

class proxy_property:
    rettype_name: str
    rettype: type
    propertyName: str
    mode: AccessMode
    def __init__(self, rettype: type, propertyName: str, mode: AccessMode) -> None: ...
    def __get__(self, instance, owner): ...
    def __set__(self, instance, value) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadMLeaderStyle(AcadObject):
    def __init__(self, obj) -> None: ...
    AlignSpace: int
    Annotative: bool
    ArrowSize: int
    ArrowSymbol: str
    BitFlags: int
    Block: str
    BlockColor: AcadAcCmColor
    BlockConnectionType: AcBlockConnectionType
    BlockRotation: float
    BlockScale: float
    BreakSize: float
    ContentType: AcMLeaderContentType
    Description: str
    DoglegLength: float
    DrawLeaderOrderType: AcDrawLeaderOrderType
    DrawMLeaderOrderType: AcDrawLeaderOrderType
    EnableBlockRotation: bool
    EnableBlockScale: bool
    EnableDogleg: bool
    EnableFrameText: bool
    EnableLanding: bool
    FirstSegmentAngleConstraint: int
    LandingGap: float
    LeaderLineColor: AcadAcCmColor
    LeaderLinetype: str
    LeaderLineTypeId: int
    LeaderLineWeight: AcLineWeight
    MaxLeaderSegmentsPoints: int
    Name: str
    OverwritePropChanged: bool
    ScaleFactor: float
    SecondSegmentAngleConstraint: int
    TextAlignmentType: AcTextAlignmentType
    TextAngleType: AcTextAngleType
    TextAttachmentDirection: AcTextAttachmentDirection
    TextBottomAttachmentType: AcVerticalTextAttachmentType
    TextColor: AcColor
    TextHeight: float
    TextLeftAttachmentType: AcTextAttachmentType
    TextRightAttachmentType: AcTextAttachmentType
    TextString: str
    TextStyle: str
    TextTopAttachmentType: AcVerticalTextAttachmentType
    def GetBoundingBox(self) -> PyGePoint3dArray: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadSpline(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Area: float = proxy_property(float,'Area',AccessMode.ReadOnly)
    Closed: bool = proxy_property(bool,'Closed',AccessMode.ReadOnly)
    Closed2: bool = proxy_property(bool,'Closed2',AccessMode.ReadWrite)
    ControlPoints: PyGePoint3dArray = proxy_property('PyGePoint3dArray','ControlPoints',AccessMode.ReadWrite)
    Degree: int = proxy_property(int,'Degree',AccessMode.ReadOnly)
    Degree2: int = proxy_property(int,'Degree2',AccessMode.ReadWrite)
    EndTangent: PyGeVector3d = proxy_property('PyGeVector3d','EndTangent',AccessMode.ReadWrite)
    FitPoints: PyGePoint3dArray = proxy_property('PyGePoint3dArray','FitPoints',AccessMode.ReadWrite)
    FitTolerance: float = proxy_property(float,'FitTolerance',AccessMode.ReadWrite)
    IsPeriodic: bool = proxy_property(bool,'IsPeriodic',AccessMode.ReadOnly)
    IsPlanar: bool = proxy_property(bool,'IsPlanar',AccessMode.ReadOnly)
    IsRational: bool = proxy_property(bool,'IsRational',AccessMode.ReadOnly)
    KnotParameterization: AcSplineKnotParameterizationType = proxy_property('AcSplineKnotParameterizationType','KnotParameterization',AccessMode.ReadWrite)
    Knots: PyGeVector3d = proxy_property('PyGeVector3d','Knots',AccessMode.ReadWrite)
    NumberOfControlPoints: int = proxy_property(int,'NumberOfControlPoints',AccessMode.ReadOnly)
    NumberOfFitPoints: int = proxy_property(int,'NumberOfFitPoints',AccessMode.ReadOnly)
    SplineFrame: AcSplineFrameType = proxy_property('AcSplineFrameType','SplineFrame',AccessMode.ReadWrite)
    SplineMethod: AcSplineMethodType = proxy_property('AcSplineMethodType','SplineMethod',AccessMode.ReadWrite)
    StartTangent: PyGeVector3d = proxy_property('PyGeVector3d','StartTangent',AccessMode.ReadWrite)
    Weights: PyGeVector3d = proxy_property('PyGeVector3d','Weights',AccessMode.ReadWrite)

    def AddFitPoint(self, Index: int, FitPoint: PyGePoint3d) -> None:
        self._obj.AddFitPoint(Index, FitPoint())
    
    def Copy(self) -> AcadSpline:
        return AcadSpline(self._obj.Copy())
        
    def DeleteFitPoint(self, Index: int) -> None:
        self._obj.DeleteFitPoint(Index)
        
    def ElevateOrder(self, Order: int) -> None:
        self._obj.ElevateOrder(Order)

    def GetControlPoint(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.GetControlPoint(Index))
        
    def GetFitPoint(self, Index: int) -> PyGePoint3d:
        return PyGePoint3d(self._obj.GetFitPoint(Index))
        
    def GetWeight(self, Index: int) -> int:
        return self._obj.GetWeight(Index)
            
    def Offset(self, Distance: float) -> AcadSpline:
        return AcadSpline(self._obj.Offset(Distance))
        
    def PurgeFitData(self) -> None:
        self._obj.PurgeFitData()
            
    def Reverse(self) -> None:
        self._obj.Reverse()
                
    def SetControlPoint(self, Index: int, Value: PyGePoint3d) -> None:
        self._obj.SetControlPoint(Index, Value)
    
    def SetFitPoint(self, Index: int, Value: PyGePoint3d) -> None:
        self._obj.SetFitPoint(Index, Value)

    def SetWeight(self, Index: int, Weight: float) -> None:
        self._obj.SetWeight(Index, Weight)



from ..Base import *
from ..Proxy import *
from .AcadBlockReference import AcadBlockReference as AcadBlockReference

class AcadMInsertBlock(AcadBlockReference):
    def __init__(self, obj) -> None: ...
    Columns: int
    ColumnSpacing: float
    def Copy(self) -> AcadMInsertBlock: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadViewport(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    ArcSmoothness: int = proxy_property(int,'ArcSmoothness',AccessMode.ReadWrite)
    Center: PyGePoint2d = proxy_property('PyGePoint2d','Center',AccessMode.ReadWrite)
    Direction: PyGeVector3d = proxy_property('PyGeVector3d','Direction',AccessMode.ReadWrite)
    GridOn: bool = proxy_property(bool,'GridOn',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    LowerLeftCorner: vDoubleArray = proxy_property('vDoubleArray','LowerLeftCorner',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    OrthoOn: bool = proxy_property(bool,'OrthoOn',AccessMode.ReadWrite)
    SnapBasePoint: vDoubleArray = proxy_property('vDoubleArray','SnapBasePoint',AccessMode.ReadWrite)
    SnapOn: bool = proxy_property(bool,'SnapOn',AccessMode.ReadWrite)
    SnapRotationAngle: float = proxy_property(float,'SnapRotationAngle',AccessMode.ReadWrite)
    Target: PyGePoint3d = proxy_property('PyGePoint3d','Target',AccessMode.ReadWrite)
    UCSIconAtOrigin: bool = proxy_property(bool,'UCSIconAtOrigin',AccessMode.ReadWrite)
    UCSIconOn: bool = proxy_property(bool,'UCSIconOn',AccessMode.ReadWrite)
    UpperRightCorner: vDoubleArray = proxy_property('vDoubleArray','UpperRightCorner',AccessMode.ReadOnly)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)

    def GetGridSpacing(self) -> vDoubleArray: 
        XSpacing, YSpacing = self._obj.GetGridSpacing()
        return vDoubleArray(XSpacing, YSpacing)
        
    def GetSnapSpacing(self) -> vDoubleArray: 
        XSpacing, YSpacing = self._obj.GetSnapSpacing()
        return vDoubleArray(XSpacing, YSpacing)
        
    def SetGridSpacing(self, XSpacing: float, YSpacing: float) -> None: 
        self._obj.SetGridSpacing(XSpacing, YSpacing)

    def SetSnapSpacing(self, XSpacing: float, YSpacing: float) -> None: 
        self._obj.SetSnapSpacing(XSpacing, YSpacing)

    def SetView(self, View: AcadView) -> None: 
        self._obj.SetView(View())

    def Split(self, NumWins: AcViewportSplitType) -> None: 
        self._obj.Split(NumWins)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadExternalReference import AcadExternalReference



class AcadComparedReference(AcadExternalReference):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Copy(self) -> AcadComparedReference:
        return AcadComparedReference(self._obj.Copy())



from ..Base import *
from ..Proxy import *
from ..AcadObject import AcadObject as AcadObject

class AcadDimStyle(AcadObject):
    def __init__(self, obj) -> None: ...
    Name: str
    def CopyFrom(self, SourceObject: AcadDimStyle | AcadDim3PointAngular | AcadDimAligned | AcadDimAngular | AcadDimArcLength | AcadDimDiametric | AcadDimOrdinate | AcadDimRadial | AcadDimRadialLarge | AcadDimRotated | AcadDocument | AcadLayout | AcadLeader | AcadPlotConfiguration) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadSelectionSet(AppObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Count: int
    Name: str
    def AddItems(self, Items: vObjectArray) -> None: ...
    def Clear(self) -> None: ...
    def Delete(self) -> None: ...
    def Erase(self) -> None: ...
    def Highlight(self, HighlightFlag: bool) -> None: ...
    def Item(self, Index: int | str) -> AcadObject: ...
    def RemoveItems(self, Objects: vObjectArray) -> None: ...
    def Select(self, Mode: AcSelect, Point1: PyGePoint3d = None, Point2: PyGePoint3d = None, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None: ...
    def SelectAtPoint(self, Point: PyGePoint3d, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None: ...
    def SelectByPolygon(self, Mode: AcSelect, PointsList: PyGePoint3dArray, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None: ...
    def SelectOnScreen(self, FilterType: Variant[AdeskDxfCode] = None, FilterData: Variant = None) -> None: ...
    def Update(self) -> None: ...



from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity as AcadEntity

class AcadMtext(AcadEntity):
    def __init__(self, obj) -> None: ...
    AttachmentPoint: AcAttachmentPoint
    BackgroundFill: bool
    DrawingDirection: AcDrawingDirection
    Height: float
    InsertionPoint: PyGePoint3d
    LineSpacingDistance: float
    LineSpacingFactor: float
    LineSpacingStyle: AcLineSpacingStyle
    Normal: PyGeVector3d
    TextString: str
    Width: float
    def FieldCode(self) -> str: ...
    def Copy(self) -> AcadMtext: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadHatch(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    Area: float = proxy_property(float,'Area',AccessMode.ReadOnly)
    AssociativeHatch: bool = proxy_property(bool,'AssociativeHatch',AccessMode.ReadOnly)
    BackgroundColor: AcadAcCmColor = proxy_property('AcadAcCmColor','BackgroundColor',AccessMode.ReadWrite)
    Elevation: float = proxy_property(float,'Elevation',AccessMode.ReadWrite)
    GradientAngle: float = proxy_property(float,'GradientAngle',AccessMode.ReadWrite)
    GradientCentered: bool = proxy_property(bool,'GradientCentered',AccessMode.ReadWrite)
    GradientColor1: AcadAcCmColor = proxy_property('AcadAcCmColor','GradientColor1',AccessMode.ReadWrite)
    GradientColor2: AcadAcCmColor = proxy_property('AcadAcCmColor','GradieGradientColor2ntCentered',AccessMode.ReadWrite)
    GradientName: str = proxy_property(str,'GradientName',AccessMode.ReadWrite)
    HatchObjectType: AcHatchObjectType = proxy_property('AcHatchObjectType','HatchObjectType',AccessMode.ReadWrite)
    HatchStyle: AcHatchStyle = proxy_property('AcHatchStyle','HatchStyle',AccessMode.ReadWrite)
    ISOPenWidth: AcISOPenWidth = proxy_property('AcISOPenWidth','ISOPenWidth',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    NumberOfLoops: int = proxy_property(int,'NumberOfLoops',AccessMode.ReadOnly)
    Origin: PyGePoint3d = proxy_property('PyGePoint3d','Origin',AccessMode.ReadWrite)
    PatternAngle: float = proxy_property(float,'PatternAngle',AccessMode.ReadWrite)
    PatternDouble: bool = proxy_property(bool,'PatternDouble',AccessMode.ReadWrite)
    PatternName: str = proxy_property(str,'PatternName',AccessMode.ReadWrite)
    PatternScale: float = proxy_property(float,'PatternScale',AccessMode.ReadWrite)
    PatternSpace: float = proxy_property(float,'PatternSpace',AccessMode.ReadWrite)
    PatternType: AcPatternType = proxy_property(AcPatternType,'PatternType',AccessMode.ReadOnly)

    def AppendInnerLoop(self, Loop: vObjectArray[AcadArc|AcadCircle|AcadEllipse|AcadLine|AcadPolyline|AcadRegion|AcadSpline]) -> None:
        self._obj.AppendInnerLoop(Loop())

    def AppendOuterLoop(self, Loop: vObjectArray[AcadArc|AcadCircle|AcadEllipse|AcadLine|AcadPolyline|AcadRegion|AcadSpline]) -> None:
        self._obj.AppendOuterLoop(Loop())



from .Autocad.AcadEntity import *
from .Autocad.AcadObject import *
from .Autocad.Entities import *
from .Autocad.Objects import *
from .Types import *
from .Autocad.Proxy import *
from abc import ABC

class AutoCAD(AcadApplication):
    def __init__(self) -> None: ...
    def uGetAcadAcCmColor(self) -> AcadAcCmColor: ...
    def uGetAcadLayerStateManager(self) -> AcadLayerStateManager: ...
    def uGetAcadSecurityParams(self) -> AcadSecurityParams: ...
    def uSetXData(obj: AcadObject, xdm: XDataManager) -> None: ...
    def uGetObjectType(self, obj: AcadObject) -> type | None: ...

class BlockReference(ABC): ...


from .Base import *
from .Proxy import *
from .AcadObject import *
from .Objects import *

class AcadEntity(AcadObject):
    def __init__(self, obj) -> None: ...
    EntityTransparency: AcEntityTransparency
    Hyperlinks: AcadHyperlinks
    Layer: str
    Linetype: str
    LinetypeScale: float
    Lineweight: AcLineWeight
    Material: str
    PlotStyleName: str
    TrueColor: AcadAcCmColor
    Visible: bool
    def ArrayPolar(self, NumberOfObjects: int, AngleToFill: float, CenterPoint: PyGePoint3d) -> vObjectArray: ...
    def ArrayRectangular(self, NumberOfRows: int, NumberOfColumns: int, NumberOfLevels: int, DistBetweenRows: float, DistBetweenColumns: float, DistBetweenLevels: float) -> vObjectArray: ...
    def Copy(self) -> AcadEntity: ...
    def GetBoundingBox(self) -> PyGePoint3dArray: ...
    def Highlight(self, HighlightFlag: bool) -> None: ...
    def IntersectWith(self, IntersectObject: AcadEntity | AcadAttributeReference, ExtendOption: AcExtendOption) -> PyGePoint3dArray: ...
    def Mirror(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> AcadEntity: ...
    def Mirror3D(self, Point1: PyGePoint3d, Point2: PyGePoint3d, Point3: PyGePoint3d) -> AcadEntity: ...
    def Move(self, Point1: PyGePoint3d, Point2: PyGePoint3d) -> None: ...
    def Rotate(self, BasePoint: PyGePoint3d, RotationAngle: float) -> None: ...
    def Rotate3D(self, Point1: PyGePoint3d, Point2: PyGePoint3d, RotationAngle: float) -> None: ...
    def ScaleEntity(self, BasePoint: PyGePoint3d, ScaleFactor: float) -> None: ...
    def TransformBy(self, TransformationMatrix: PyGeMatrix3d) -> None: ...
    def Update(self) -> None: ...

class AcadSubEntity(AppObject):
    def __init__(self, obj) -> None: ...
    Color: AcColor
    Hyperlinks: AcadHyperlinks
    Layer: str
    Linetype: str
    LinetypeScale: float
    Lineweight: AcLineWeight
    ObjectName: str
    PlotStyleName: str



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadPreferencesProfiles(AppObject):
    def __init__(self, obj) -> None: ...
    ActiveProfile: str
    Application: AcadApplication
    def CopyProfile(self, oldProfileName: str, newProfileName: str) -> None: ...
    def DeleteProfile(self, ProfileName: str) -> None: ...
    def ExportProfile(self, Profile: str, RegFile: str) -> None: ...
    def GetAllProfileNames(self) -> tuple[str]: ...
    def ImportProfile(self, Profile: str, RegFile: str, IncludePathInfo: bool) -> None: ...
    def RenameProfile(self, origProfileName: str, newProfileName: str) -> None: ...
    def ResetProfile(self, Profile: str) -> None: ...



from ..Base import *
from ..Proxy import *
from .AcadDimension import AcadDimension as AcadDimension

class AcadDimAngular(AcadDimension):
    def __init__(self, obj) -> None: ...
    AngleFormat: AcAngleUnits
    AngleVertex: PyGePoint3d
    Arrowhead1Block: str
    Arrowhead1Type: AcDimArrowheadType
    Arrowhead2Block: str
    Arrowhead2Type: AcDimArrowheadType
    ArrowheadSize: int
    DimConstrDesc: str
    DimConstrExpression: str
    DimConstrForm: bool
    DimConstrName: str
    DimConstrReference: bool
    DimConstrValue: str
    DimensionLineColor: AcColor
    DimensionLinetype: str
    DimensionLineWeight: AcLineWeight
    DimLine1Suppress: bool
    DimLine2Suppress: bool
    DimLineInside: bool
    ExtensionLineColor: AcColor
    ExtensionLineExtend: float
    ExtensionLineOffset: float
    ExtensionLineWeight: AcLineWeight
    ExtLine1EndPoint: PyGePoint3d
    ExtLine1Linetype: str
    ExtLine1StartPoint: PyGePoint3d
    ExtLine1Suppress: bool
    ExtLine2EndPoint: PyGePoint3d
    ExtLine2Linetype: str
    ExtLine2StartPoint: PyGePoint3d
    ExtLine2Suppress: bool
    ExtLineFixedLen: float
    ExtLineFixedLenSuppress: bool
    Fit: AcDimFit
    ForceLineInside: bool
    HorizontalTextPosition: AcDimHorizontalJustification
    Measurement: float
    TextInside: bool
    TextInsideAlign: bool
    TextOutsideAlign: bool
    TextPrecision: AcDimPrecision
    def Copy(self) -> AcadDimAngular: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *



class AcadPreferencesSelection(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    DisplayGrips: bool = proxy_property(bool,'DisplayGrips',AccessMode.ReadWrite)
    DisplayGripsWithinBlocks: bool = proxy_property(bool,'DisplayGripsWithinBlocks',AccessMode.ReadWrite)
    GripColorSelected: AcColor = proxy_property('AcColor','GripColorSelected',AccessMode.ReadWrite)
    GripColorUnselected: AcColor = proxy_property('AcColor','GripColorUnselected',AccessMode.ReadWrite)
    GripSize: int = proxy_property(int,'GripSize',AccessMode.ReadWrite)
    PickAdd: bool = proxy_property(bool,'PickAdd',AccessMode.ReadWrite)
    PickAuto: bool = proxy_property(bool,'PickAuto',AccessMode.ReadWrite)
    PickBoxSize: int = proxy_property(int,'PickBoxSize',AccessMode.ReadWrite)
    PickDrag: bool = proxy_property(bool,'PickDrag',AccessMode.ReadWrite)
    PickFirst: bool = proxy_property(bool,'PickFirst',AccessMode.ReadWrite)
    PickGroup: bool = proxy_property(bool,'PickGroup',AccessMode.ReadWrite)


from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadState(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)
    
    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    IsQuiescent: bool = proxy_property(bool,'IsQuiescent',AccessMode.ReadOnly)



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlock import IAcadBlock as IAcadBlock

class AcadPaperSpace(IAcadBlock):
    def __init__(self, obj) -> None: ...
    Name: str
    def AddPViewport(self, Center: PyGePoint3d, Width: float, Height: float) -> AcadPViewport: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *



class AcadView(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    CategoryName: str = proxy_property(str,'CategoryName',AccessMode.ReadWrite)
    Center: PyGePoint2d = proxy_property('PyGePoint2d','Center',AccessMode.ReadWrite)
    Direction: PyGeVector3d = proxy_property('PyGeVector3d','Direction',AccessMode.ReadWrite)
    HasVpAssociation: bool = proxy_property(bool,'HasVpAssociation',AccessMode.ReadWrite)
    Height: float = proxy_property(float,'Height',AccessMode.ReadWrite)
    LayerState: str = proxy_property(str,'LayerState',AccessMode.ReadWrite)
    LayoutID: int = proxy_property(int,'LayoutID',AccessMode.ReadOnly)
    Name: str = proxy_property(str,'Name',AccessMode.ReadWrite)
    Target: PyGePoint3d = proxy_property('PyGePoint3d','Target',AccessMode.ReadWrite)
    Width: float = proxy_property(float,'Width',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity



class AcadDimension(AcadEntity):
    def __init__(self, obj) -> None: super().__init__(obj)

    DecimalSeparator: str = proxy_property(str,'DecimalSeparator',AccessMode.ReadWrite)
    DimTxtDirection: bool = proxy_property(bool,'DimTxtDirection',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    Rotation: float = proxy_property(float,'Rotation',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    StyleName: str = proxy_property(str,'StyleName',AccessMode.ReadWrite)
    SuppressLeadingZeros: bool = proxy_property(bool,'SuppressLeadingZeros',AccessMode.ReadWrite)
    SuppressTrailingZeros: bool = proxy_property(bool,'SuppressLeadingZeros',AccessMode.ReadWrite)
    TextColor: AcColor = proxy_property('AcColor','TextColor',AccessMode.ReadWrite)
    TextFill: bool = proxy_property(bool,'TextFill',AccessMode.ReadWrite)
    TextFillColor: AcColor = proxy_property('AcColor','TextFillColor',AccessMode.ReadWrite)
    TextGap: float = proxy_property(float,'TextGap',AccessMode.ReadWrite)
    TextHeight: float = proxy_property(float,'TextHeight',AccessMode.ReadWrite)
    TextMovement: AcDimTextMovement = proxy_property('AcDimTextMovement','TextMovement',AccessMode.ReadWrite)
    TextOverride: str = proxy_property(str,'TextOverride',AccessMode.ReadWrite)
    TextPosition: PyGePoint3d = proxy_property('PyGePoint3d','TextPosition',AccessMode.ReadWrite)
    TextPrefix: str = proxy_property(str,'TextPrefix',AccessMode.ReadWrite)
    TextRotation: float = proxy_property(float,'TextRotation',AccessMode.ReadWrite)
    TextStyle: str = proxy_property(str,'TextStyle',AccessMode.ReadWrite)
    TextSuffix: str = proxy_property(str,'TextSuffix',AccessMode.ReadWrite)
    ToleranceDisplay: AcDimToleranceMethod = proxy_property('AcDimToleranceMethod','ToleranceDisplay',AccessMode.ReadWrite)
    ToleranceHeightScale: float = proxy_property(float,'ToleranceHeightScale',AccessMode.ReadWrite)
    ToleranceJustification: AcDimToleranceJustify = proxy_property('AcDimToleranceJustify','ToleranceJustification',AccessMode.ReadWrite)
    ToleranceLowerLimit: float = proxy_property(float,'ToleranceLowerLimit',AccessMode.ReadWrite)
    TolerancePrecision: AcDimPrecision = proxy_property('AcDimPrecision','ToleranceLowerLimit',AccessMode.ReadWrite)
    ToleranceSuppressLeadingZeros: bool = proxy_property(bool,'ToleranceSuppressLeadingZeros',AccessMode.ReadWrite)
    ToleranceSuppressTrailingZeros: bool = proxy_property(bool,'ToleranceSuppressTrailingZeros',AccessMode.ReadWrite)
    ToleranceUpperLimit: float = proxy_property(float,'ToleranceUpperLimit',AccessMode.ReadWrite)
    VerticalTextPosition: AcDimVerticalJustification = proxy_property('AcDimVerticalJustification','VerticalTextPosition',AccessMode.ReadWrite)

    def Copy(self) -> AcadDimension:
        return AcadDimension(self._obj.Copy())



        





from __future__ import annotations
from ..Base import *
from ..Proxy import *
from .AcadApplication import AcadApplication
from ...Types.Ac import *



class AcadSecurityParams(AppObject):
    def __init__(self, obj) -> None: 
        super().__init__(obj)

    Action: AcadSecurityParamsType = proxy_property('AcadSecurityParamsType','Action',AccessMode.ReadWrite)
    Algorithm: AcadSecurityParamsConstants = proxy_property('AcadSecurityParamsConstants','Algorithm',AccessMode.ReadWrite)
    Comment: str = proxy_property(str,'Comment',AccessMode.ReadWrite)
    Issuer: str = proxy_property(str,'Issuer',AccessMode.ReadWrite)
    KeyLength: int = proxy_property(int,'KeyLength',AccessMode.ReadWrite)
    Password: str = proxy_property(str,'Password',AccessMode.ReadWrite)
    ProviderName: str = proxy_property(str,'ProviderName',AccessMode.ReadWrite)
    ProviderType: int = proxy_property(int,'ProviderType',AccessMode.ReadWrite)
    SerialNumber: str = proxy_property(str,'SerialNumber',AccessMode.ReadWrite)
    Subject: str = proxy_property(str,'Subject',AccessMode.ReadWrite)
    TimeServer: str = proxy_property(str,'TimeServer',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *



class AcadDynamicBlockReferenceProperty(AppObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    AllowedValues: any = proxy_property(any,'AllowedValues',AccessMode.ReadOnly)
    Description: str = proxy_property(str,'Description',AccessMode.ReadOnly)
    PropertyName: str = proxy_property(str,'PropertyName',AccessMode.ReadOnly)
    ReadOnly: bool = proxy_property(bool,'ReadOnly',AccessMode.ReadOnly)
    Show: bool = proxy_property(bool,'Show',AccessMode.ReadOnly)
    UnitsType: AcDynamicBlockReferencePropertyUnitsType  = proxy_property('AcDynamicBlockReferencePropertyUnitsType','UnitsType',AccessMode.ReadOnly)
    Value: any  = proxy_property(any,'Value',AccessMode.ReadWrite)



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadEntity import AcadEntity
from .AcadLeader import AcadLeader


class AcadMLeader(AcadEntity):
    def __init__(self, obj, leaderLineIndex=0) -> None: 
        self.leaderLineIndex = leaderLineIndex
        super().__init__(obj)

    ArrowheadBlock: str = proxy_property(str,'ArrowheadBlock',AccessMode.ReadWrite)
    ArrowheadSize: float = proxy_property(float,'ArrowheadSize',AccessMode.ReadWrite)
    ArrowheadType: AcDimArrowheadType = proxy_property(float,'ArrowheadType',AccessMode.ReadWrite)
    BlockConnectionType: AcBlockConnectionType = proxy_property('AcBlockConnectionType','BlockConnectionType',AccessMode.ReadWrite)
    BlockScale: int = proxy_property(int,'BlockScale',AccessMode.ReadWrite)
    ContentBlockName: str = proxy_property(str,'ContentBlockName',AccessMode.ReadWrite)
    ContentBlockType: AcPredefBlockType = proxy_property('AcPredefBlockType','ContentBlockType',AccessMode.ReadWrite)
    ContentType: AcMLeaderContentType = proxy_property('AcMLeaderContentType','ContentType',AccessMode.ReadWrite)
    DogLegged: bool = proxy_property(bool,'DogLegged',AccessMode.ReadWrite)
    DoglegLength: float = proxy_property(float,'DoglegLength',AccessMode.ReadWrite)
    LandingGap: float = proxy_property(float,'LandingGap',AccessMode.ReadWrite)
    LeaderCount: int = proxy_property(int,'LeaderCount',AccessMode.ReadOnly)
    LeaderLineColor: AcadAcCmColor = proxy_property('AcadAcCmColor','LeaderLineColor',AccessMode.ReadWrite)
    LeaderLinetype: str = proxy_property(str,'LeaderLinetype',AccessMode.ReadWrite)
    LeaderLineWeight: AcLineWeight = proxy_property('AcLineWeight','LeaderLineWeight',AccessMode.ReadWrite)
    LeaderType: AcMLeaderType = proxy_property('AcMLeaderType','LeaderType',AccessMode.ReadWrite)
    Normal: PyGeVector3d = proxy_property('PyGeVector3d','Normal',AccessMode.ReadWrite)
    ScaleFactor: float = proxy_property(float,'ScaleFactor',AccessMode.ReadWrite)
    StyleName: str = proxy_property(str,'StyleName',AccessMode.ReadWrite)
    TextAttachmentDirection: AcTextAttachmentDirection = proxy_property('AcTextAttachmentDirection','TextAttachmentDirection',AccessMode.ReadWrite)
    TextBackgroundFill: bool = proxy_property(bool,'TextBackgroundFill',AccessMode.ReadWrite)
    TextBottomAttachmentType: AcVerticalTextAttachmentType = proxy_property('AcVerticalTextAttachmentType','TextBottomAttachmentType',AccessMode.ReadWrite)
    TextDirection: AcDrawingDirection = proxy_property('AcDrawingDirection','TextDirection',AccessMode.ReadWrite)
    TextFrameDisplay: bool = proxy_property(bool,'TextFrameDisplay',AccessMode.ReadWrite)
    TextHeight: float = proxy_property(float,'TextHeight',AccessMode.ReadWrite)
    TextJustify: AcAttachmentPoint = proxy_property('AcAttachmentPoint','TextJustify',AccessMode.ReadWrite)
    TextLeftAttachmentType: AcTextAttachmentType = proxy_property('AcTextAttachmentType','TextLeftAttachmentType',AccessMode.ReadWrite)
    TextLineSpacingDistance: float = proxy_property(float,'TextLineSpacingDistance',AccessMode.ReadWrite)
    TextLineSpacingFactor: float = proxy_property(float,'TextLineSpacingFactor',AccessMode.ReadWrite)
    TextLineSpacingStyle: AcLineSpacingStyle = proxy_property('AcLineSpacingStyle','TextLineSpacingStyle',AccessMode.ReadWrite)
    TextRightAttachmentType: AcTextAttachmentType = proxy_property('AcTextAttachmentType','TextRightAttachmentType',AccessMode.ReadWrite)
    TextRotation: float = proxy_property(float,'TextRotation',AccessMode.ReadWrite)
    TextString: str = proxy_property(str,'TextString',AccessMode.ReadWrite)
    TextStyleName: str = proxy_property(str,'TextStyleName',AccessMode.ReadWrite)
    TextTopAttachmentType: AcVerticalTextAttachmentType = proxy_property('AcVerticalTextAttachmentType','TextTopAttachmentType',AccessMode.ReadWrite)
    TextWidth: float = proxy_property(float,'TextWidth',AccessMode.ReadWrite)
    Type: AcLeaderType = proxy_property('AcLeaderType','Type',AccessMode.ReadWrite)

    def AddLeader(self) -> AcadLeader:
        return AcadLeader(self._obj.AddLeader())

    def AddLeaderLine(self, leaderIndex: int, pointArray: PyGePoint3dArray) -> int:
        return self._obj.AddLeaderLine(leaderIndex, pointArray())

    def AddLeaderLineEx(self, pointArray: PyGePoint3dArray) -> int:
        return self._obj.AddLeaderLineEx(pointArray())

    def Evaluate(self) -> None:
        self._obj.Evaluate()
    
    def GetBlockAttributeValue(self, attdefId: int) -> str:
        return self._obj.GetBlockAttributeValue(attdefId)

    def GetDoglegDirection(self, leaderIndex: int) -> PyGeVector3d:
        return PyGeVector3d(self._obj.GetDoglegDirection(leaderIndex))
    
    def GetLeaderIndex(self, leaderLineIndex: int) -> int:
        return self._obj.GetLeaderIndex(leaderLineIndex)
        
    def GetLeaderLineIndexes(self, leaderIndex: int) -> vDoubleArray:
        return vDoubleArray(self._obj.GetLeaderLineIndexes(leaderIndex))

    def GetLeaderLineVertices(self, leaderLineIndex: int) -> PyGePoint3dArray:
        return PyGePoint3dArray(self._obj.GetLeaderLineVertices(leaderLineIndex))

    def GetVertexCount(self, leaderLineIndex: int) -> int:
        return self._obj.GetVertexCount(leaderLineIndex)
    
    def RemoveLeader(self, leaderIndex: int) -> None:
        self._obj.RemoveLeader(leaderIndex)

    def RemoveLeaderLine(self, leaderLineIndex: int) -> None:
        self._obj.RemoveLeaderLine(leaderLineIndex)

    def SetBlockAttributeValue(self, attdefId: int, value: str) -> None:
        self._obj.SetBlockAttributeValue(attdefId, value)

    def SetDoglegDirection(self, leaderIndex: int, dirVec: vDoubleArray) -> None:
        self._obj.SetDoglegDirection(leaderIndex, dirVec())
        
    def SetLeaderLineVertices(self, leaderLineIndex: int, pointArray: PyGePoint3dArray) -> None:
        self._obj.SetLeaderLineVertices(leaderLineIndex, pointArray())



from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadBlocks import AcadBlocks as AcadBlocks
from .AcadDatabasePreferences import AcadDatabasePreferences as AcadDatabasePreferences
from .AcadDictionaries import AcadDictionaries as AcadDictionaries
from .AcadDimStyles import AcadDimStyles as AcadDimStyles
from .AcadGroups import AcadGroups as AcadGroups
from .AcadLayers import AcadLayers as AcadLayers
from .AcadLayouts import AcadLayouts as AcadLayouts
from .AcadLineTypes import AcadLineTypes as AcadLineTypes
from .AcadMaterials import AcadMaterials as AcadMaterials
from .AcadModelSpace import AcadModelSpace as AcadModelSpace
from .AcadPaperSpace import AcadPaperSpace as AcadPaperSpace
from .AcadPlotConfigurations import AcadPlotConfigurations as AcadPlotConfigurations
from .AcadRegisteredApplications import AcadRegisteredApplications as AcadRegisteredApplications
from .AcadSectionManager import AcadSectionManager as AcadSectionManager
from .AcadSummaryInfo import AcadSummaryInfo as AcadSummaryInfo
from .AcadTextStyles import AcadTextStyles as AcadTextStyles
from .AcadUCSs import AcadUCSs as AcadUCSs
from .AcadViewports import AcadViewports as AcadViewports
from .AcadViews import AcadViews as AcadViews

class AcadDatabase(AppObject):
    def __init__(self, obj) -> None: ...
    Blocks: AcadBlocks
    Dictionaries: AcadDictionaries
    DimStyles: AcadDimStyles
    ElevationModelSpace: float
    ElevationPaperSpace: float
    Groups: AcadGroups
    Layers: AcadLayers
    Layouts: AcadLayouts
    Limits: vDoubleArray
    Linetypes: AcadLineTypes
    Materials: AcadMaterials
    ModelSpace: AcadModelSpace
    PaperSpace: AcadPaperSpace
    PlotConfigurations: AcadPlotConfigurations
    Preferences: AcadDatabasePreferences
    RegisteredApplications: AcadRegisteredApplications
    SectionManager: AcadSectionManager
    SummaryInfo: AcadSummaryInfo
    TextStyles: AcadTextStyles
    UserCoordinateSystems: AcadUCSs
    Viewports: AcadViewports
    Views: AcadViews
    def CopyObjects(self, Objects: vObjectArray, Owner: vObject = None) -> vObjectArray: ...
    def HandleToObject(self, Handle: str) -> AppObject: ...
    def ObjectIdToObject(self, ID: int) -> AppObject: ...

class IAcadDatabase(AcadDatabase):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadLayers(IAcadObjectCollection):
    def __init__(self, obj) -> None: ...
    def Add(self, Name: str) -> AcadLayer: ...
    def Item(self, Index: int | str) -> AcadLayer: ...
    def __iter__(self): ...



from __future__ import annotations
from ..Base import *
from ..Proxy import proxy_property, AccessMode
from ..AcadObject import *
from .AcadViewport import AcadViewport



class AcadViewports(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Name: str) -> AcadViewport: 
        return AcadViewport(self._obj.Add(Name))

    def DeleteConfiguration (self, Name: str) -> None: 
        self._obj.DeleteConfiguration(Name)

    def Item(self, Index: int | str) -> AcadViewport:
        return AcadViewport(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadViewport(item)
            yield obj



from __future__ import annotations
from ..Proxy import *
from ..AcadObject import AcadObject
from .AcadXRecord import AcadXRecord


class AcadDictionary(AcadObject):
    def __init__(self, obj) -> None: super().__init__(obj)

    Name: str = proxy_property(str,'Name',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)

    def AddObject(self, Keyword: str, ObjectName: str) -> AcadObject: 
        return AcadObject(self._obj.AddObject(Keyword, ObjectName))

    def AddXRecord(self, Keyword: str) -> AcadXRecord: 
        return AcadXRecord(self._obj.AddXRecord(Keyword))

    def GetName(self, Object: AppObject) -> str: 
        return self._obj.GetName(Object())

    def GetObject(self, Name: str) -> AcadObject: 
        return AcadObject(self._obj.GetObject(Name))

    def Item(self, Index: int | str) -> AcadObject: 
        return AcadObject(self._obj.Item(Index))

    def Remove(self, Name: str) -> AcadObject: 
        return AcadObject(self._obj.Remove(Name))

    def Rename(self, OldName: str, NewName: str) -> None: 
        self._obj.Rename(OldName, NewName)
        
    def Replace(self, Name: str, NewObject: AppObject) -> None: 
        self._obj.Replace(Name, NewObject())



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..AcadObject import *
from .AcadUCS import AcadUCS



class AcadUCSs(IAcadObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    def Add(self, Origin: PyGePoint3d, XAxisPoint: PyGePoint3d, YAxisPoint: PyGePoint3d, Name: str) -> AcadUCS: 
        return AcadUCS(self._obj.Add(Origin(), XAxisPoint(), YAxisPoint(), Name))

    def Item(self, Index: int) -> AcadUCS:
        return AcadUCS(self._obj.Item(Index))

    def __iter__(self):
        for item in self._obj:
            obj = AcadUCS(item)
            yield obj



from ..Base import *
from ..Proxy import *
from ..AcadObject import *

class AcadLayer(AcadObject):
    def __init__(self, obj) -> None: ...
    Application: AcadApplication
    Description: str
    Document: AcadDocument
    Freeze: bool
    Handle: int
    HasExtensionDictionary: bool
    LayerOn: bool
    Linetype: str
    Lineweight: AcLineWeight



from ..Base import *
from ..AcadObject import *
from ..Proxy import AccessMode as AccessMode, proxy_property as proxy_property

class AcadLayerStateManager(AppObject):
    def __init__(self, obj) -> None: ...
    def Mask(self, Name: str) -> AcLayerStateMask | int: ...
    def Delete(self, Name: str) -> None: ...
    def Export(self, Name: str, FileName: str) -> None: ...
    def Import(self, FileName: str) -> None: ...
    def Rename(self, OldName: str, NewName: str) -> None: ...
    def Restore(self, Name: str) -> None: ...
    def Save(self, Name: str, Mask: AcLayerStateMask | int) -> None: ...
    def SetDatabase(self, Database: AcadDatabase) -> None: ...



from __future__ import annotations
from ..Base import *
from ..Proxy import *
from ..Objects.AcadApplication import AcadApplication


class AcadMenuBar(AppObjectCollection):
    def __init__(self, obj) -> None: super().__init__(obj)

    Application: AcadApplication = proxy_property('AcadApplication','Application',AccessMode.ReadOnly)
    Count: int = proxy_property(int,'Count',AccessMode.ReadOnly)
    Parent: AppObject = proxy_property('AppObject','Parent',AccessMode.ReadWrite)

